@@ -1,5 +1,5 @@
 // Copyright (c) Facebook, Inc. and its affiliates.
-// SPDX-License-Identifier: GPL-3.0+
+// SPDX-License-Identifier: GPL-3.0-or-later
 
 /**
  * @file
@@ -24,13 +24,9 @@
  * @{
  */
 
-struct drgn_object_type;
+struct drgn_operand_type;
 
-/**
- * Global stop iteration error.
- *
- * This is also used as a special sentinel return in various places internally.
- */
+/** Global stop iteration error. */
 extern struct drgn_error drgn_stop;
 
 /** Global @ref DRGN_ERROR_OBJECT_ABSENT error. */
@@ -57,15 +53,15 @@ bool string_builder_append_error(struct string_builder *sb,
 
 /** Create a @ref drgn_error from the libelf error indicator. */
 struct drgn_error *drgn_error_libelf(void)
-	__attribute__((returns_nonnull));
+	__attribute__((__returns_nonnull__));
 
 /** Create a @ref drgn_error from the libdw error indicator. */
 struct drgn_error *drgn_error_libdw(void)
-	__attribute__((returns_nonnull));
+	__attribute__((__returns_nonnull__));
 
 /** Create a @ref drgn_error from the libdwfl error indicator. */
 struct drgn_error *drgn_error_libdwfl(void)
-	__attribute__((returns_nonnull));
+	__attribute__((__returns_nonnull__));
 
 /**
  * Create a @ref drgn_error with a type name.
@@ -76,7 +72,7 @@ struct drgn_error *drgn_error_libdwfl(void)
  * will be replaced with the type name, and no other conversion specifications.
  */
 struct drgn_error *drgn_type_error(const char *format, struct drgn_type *type)
-	__attribute__((returns_nonnull));
+	__attribute__((__returns_nonnull__));
 
 /**
  * Create a @ref drgn_error with a qualified type name.
@@ -86,7 +82,7 @@ struct drgn_error *drgn_type_error(const char *format, struct drgn_type *type)
 struct drgn_error *
 drgn_qualified_type_error(const char *format,
 			  struct drgn_qualified_type qualified_type)
-	__attribute__((returns_nonnull));
+	__attribute__((__returns_nonnull__));
 
 /**
  * Create a @ref drgn_error for an incomplete type.
@@ -98,18 +94,18 @@ struct drgn_error *drgn_error_incomplete_type(const char *format,
 
 /** Create a @ref drgn_error for invalid types to a binary operator. */
 struct drgn_error *drgn_error_binary_op(const char *op_name,
-					struct drgn_object_type *type1,
-					struct drgn_object_type *type2)
-	__attribute__((returns_nonnull));
+					struct drgn_operand_type *type1,
+					struct drgn_operand_type *type2)
+	__attribute__((__returns_nonnull__));
 
 /** Create a @ref drgn_error for an invalid type to a unary operator. */
 struct drgn_error *drgn_error_unary_op(const char *op_name,
-				       struct drgn_object_type *type)
-	__attribute__((returns_nonnull));
+				       struct drgn_operand_type *type)
+	__attribute__((__returns_nonnull__));
 
 /** Create a @ref drgn_error for a failed symbol lookup. */
 struct drgn_error *drgn_error_symbol_not_found(uint64_t address)
-	__attribute__((returns_nonnull));
+	__attribute__((__returns_nonnull__));
 
 /** @} */
 
