@@ -25,23 +25,24 @@
  * @ref drgn_memory_reader provides a common interface for registering regions
  * of memory in a program and reading from memory.
  *
+ * @ref drgn_memory_reader does not have a notion of the maximum address or
+ * address overflow/wrap-around. Those must be handled at a higher layer.
+ *
  * @{
  */
 
 /** Memory segment in a @ref drgn_memory_reader. */
 struct drgn_memory_segment {
 	struct binary_tree_node node;
-	/** Address of the segment in memory. */
-	uint64_t address;
-	/** Size of the segment in bytes; */
-	uint64_t size;
+	/** Address range of the segment in memory (inclusive). */
+	uint64_t min_address, max_address;
 	/**
 	 * The address of the segment when it was added, before any truncations.
 	 *
-	 * This is always greater than or equal to @ref
-	 * drgn_memory_segment::address.
+	 * This is always less than or equal to @ref
+	 * drgn_memory_segment::min_address.
 	 */
-	uint64_t orig_address;
+	uint64_t orig_min_address;
 	/** Read callback. */
 	drgn_memory_read_fn read_fn;
 	/** Argument to pass to @ref drgn_memory_segment::read_fn. */
@@ -51,7 +52,7 @@ struct drgn_memory_segment {
 static inline uint64_t
 drgn_memory_segment_to_key(const struct drgn_memory_segment *entry)
 {
-	return entry->address;
+	return entry->min_address;
 }
 
 DEFINE_BINARY_SEARCH_TREE_TYPE(drgn_memory_segment_tree,
@@ -84,10 +85,20 @@ void drgn_memory_reader_deinit(struct drgn_memory_reader *reader);
 /** Return whether a @ref drgn_memory_reader has no segments. */
 bool drgn_memory_reader_empty(struct drgn_memory_reader *reader);
 
-/** @sa drgn_program_add_memory_segment() */
+/**
+ * Add a segment to a @ref drgn_memory_reader.
+ *
+ * @param[in] reader Memory reader.
+ * @param[in] min_address Start address (inclusive).
+ * @param[in] max_address End address (inclusive). Must be `>= min_address`.
+ * @param[in] read_fn Callback to read from segment.
+ * @param[in] arg Argument to pass to @p read_fn.
+ * @param[in] physical Whether to add a physical memory segment.
+ * @return @c NULL on success, non-@c NULL on error.
+ */
 struct drgn_error *
 drgn_memory_reader_add_segment(struct drgn_memory_reader *reader,
-			       uint64_t address, uint64_t size,
+			       uint64_t min_address, uint64_t max_address,
 			       drgn_memory_read_fn read_fn, void *arg,
 			       bool physical);
 
@@ -97,7 +108,8 @@ drgn_memory_reader_add_segment(struct drgn_memory_reader *reader,
  * @param[in] reader Memory reader.
  * @param[out] buf Buffer to read into.
  * @param[in] address Starting address in memory to read.
- * @param[in] count Number of bytes to read.
+ * @param[in] count Number of bytes to read. `address + count - 1` must be
+ * `<= UINT64_MAX`
  * @param[in] physical Whether @c address is physical.
  * @return @c NULL on success, non-@c NULL on error.
  */
