@@ -1,5 +1,5 @@
 // Copyright (c) Facebook, Inc. and its affiliates.
-// SPDX-License-Identifier: GPL-3.0+
+// SPDX-License-Identifier: GPL-3.0-or-later
 
 #include <stdarg.h>
 
@@ -54,44 +54,6 @@ PyObject *join_strings(PyObject *parts)
 	return ret;
 }
 
-PyObject *byteorder_string(bool little_endian)
-{
-	_Py_IDENTIFIER(little);
-	_Py_IDENTIFIER(big);
-	PyObject *ret;
-
-	ret = _PyUnicode_FromId(little_endian ? &PyId_little : &PyId_big);
-	Py_XINCREF(ret);
-	return ret;
-}
-
-int byteorder_converter(PyObject *o, void *p)
-{
-	struct byteorder_arg *arg = p;
-
-	arg->is_none = o == Py_None;
-	if (arg->allow_none && o == Py_None)
-		return 1;
-
-	if (PyUnicode_Check(o)) {
-		const char *s;
-
-		s = PyUnicode_AsUTF8(o);
-		if (strcmp(s, "little") == 0) {
-			arg->value = DRGN_LITTLE_ENDIAN;
-			return 1;
-		} else if (strcmp(s, "big") == 0) {
-			arg->value = DRGN_BIG_ENDIAN;
-			return 1;
-		}
-	}
-	PyErr_Format(PyExc_ValueError,
-		     "expected 'little'%s 'big'%s for byteorder",
-		     arg->allow_none ? "," : " or",
-		     arg->allow_none ? ", or None" : "");
-	return 0;
-}
-
 int index_converter(PyObject *o, void *p)
 {
 	struct index_arg *arg = p;
