@@ -1,6 +1,7 @@
 # Copyright (c) Facebook, Inc. and its affiliates.
-# SPDX-License-Identifier: GPL-3.0+
+# SPDX-License-Identifier: GPL-3.0-or-later
 
+import contextlib
 import ctypes
 import errno
 import os
@@ -151,3 +152,19 @@ def create_socket(*args, **kwds):
             raise unittest.SkipTest("kernel does not support TCP")
         else:
             raise
+
+
+@contextlib.contextmanager
+def setenv(key, value):
+    old_value = os.environ.get(key)
+    try:
+        if value is not None:
+            os.environ[key] = value
+        elif old_value is not None:
+            del os.environ[key]
+        yield
+    finally:
+        if old_value is None:
+            del os.environ[key]
+        else:
+            os.environ[key] = old_value
